/* nologin.c

   Program to be used as shell for users whose login privileges have
   been suspended (e.g. for cracked password).  It issues a message
   on the terminal and exits, logging the attempt with syslog.  It
   can also be used as shell for non-login accounts like bin and adm.

   Author:  Robert Moniot <moniot@fordham.edu>
	    with ideas from Brad Powell, Titan noshell.c
   Date:    5 Aug 1998
   Version: 1.1
*/

#include <stdio.h>
#include <syslog.h>
#include <unistd.h>
#include <pwd.h> 
#include <sys/types.h>
#include <signal.h>

	/* Here is the message to be given to the poor luser.   Customize
	   it by editing here, or just define CONTACT on the compile line, e.g.
	   cc -DCONTACT='"  Charles Babbage 987-6543"' nologin.c -o nologin
	*/

char message[]="\n"
"**********************************************************************\n"
"\n"
"Your login privileges on this machine have been temporarily suspended.\n"
"Please contact the system administrator by phone for further\n"
"information.\n"
#ifdef CONTACT
CONTACT
#endif
"\n"
"We apologize for the inconvenience.\n"
"\n"
"**********************************************************************\n"
;


main(int argc, char *argv[])
{
  int i, fd_msg;
  extern char **environ;
				/* Get uid and then look up user name
				   from that, for log message.
				*/
  int uid = getuid();
  struct passwd *pwent;
  pwent = getpwuid(uid);

				/* Clean out the environment.
				 */
  if (environ != NULL)
    environ[0] = NULL;

				/* Disable the signals while we log
				   the attempt.
				*/
  for (i = 1; i < NSIG; i++)
    (void) signal(i, SIG_IGN);

				/* Log the attempt.  We use LOG_AUTH
				   for compatibility, although the linux
				   manual recommends LOG_AUTHPRIV.
				*/
  openlog(argv[0],0,LOG_AUTH);
  if(pwent != NULL)
    syslog(LOG_WARNING,"attempted login by %s",pwent->pw_name);
  else
    syslog(LOG_WARNING,"attempted login by %d",uid);

  closelog();

				/* Print a message to the user. */
  
  write(1,message,sizeof(message)-1);

				/* Re-enable some signals so we can be
				   killed while we sleep.
				*/
  (void) signal(SIGHUP, SIG_DFL);
  (void) signal(SIGINT, SIG_DFL);
  (void) signal(SIGQUIT, SIG_DFL);
  (void) signal(SIGTERM, SIG_DFL);

				/* Wait 5 seconds in case this screen
				   vanishes on exit. */
  (void) sleep(5);


  return 0;			/* Exit status is ignored; 0 is fine. */
}

